From: Mike Waychison <mikew@google.com>
To: simon.kagstrom@netinsight.net, davem@davemloft.net,
nhorman@tuxdriver.com, Matt Mackall <mpm@selenic.com>
Cc: adurbin@google.com, linux-kernel@vger.kernel.org,
chavey@google.com, "Greg KH" <greg@kroah.com>,
"Américo Wang" <xiyou.wangcong@gmail.com>,
akpm@linux-foundation.org, linux-api@vger.kernel.org
Subject: [PATCH v2 05/23] netconsole: Wrap the list and locking in a structure
Date: Mon, 08 Nov 2010 12:32:06 -0800 [thread overview]
Message-ID: <20101108203205.22479.97761.stgit@crlf.mtv.corp.google.com> (raw)
In-Reply-To: <20101108203120.22479.19708.stgit@crlf.mtv.corp.google.com>
In preparation for moving all the netpoll target handling code out of
netconsole proper and into netpoll, wrap the list and lock in a new
structure, netpoll_targets.
Signed-off-by: Mike Waychison <mikew@google.com>
---
drivers/net/netconsole.c | 85 ++++++++++++++++++++++++----------------------
1 files changed, 44 insertions(+), 41 deletions(-)
diff --git a/drivers/net/netconsole.c b/drivers/net/netconsole.c
index 02ba5c4..f28681b 100644
--- a/drivers/net/netconsole.c
+++ b/drivers/net/netconsole.c
@@ -65,11 +65,14 @@ static int __init option_setup(char *opt)
__setup("netconsole=", option_setup);
#endif /* MODULE */
-/* Linked list of all configured targets */
-static LIST_HEAD(target_list);
-
-/* This needs to be a spinlock because write_msg() cannot sleep */
-static DEFINE_SPINLOCK(target_list_lock);
+struct netpoll_targets {
+ struct list_head list;
+ spinlock_t lock;
+};
+#define DEFINE_NETPOLL_TARGETS(x) struct netpoll_targets x = \
+ { .list = LIST_HEAD_INIT(x.list), \
+ .lock = __SPIN_LOCK_UNLOCKED(x.lock) }
+static DEFINE_NETPOLL_TARGETS(targets);
#define NETPOLL_DISABLED 0
#define NETPOLL_SETTINGUP 1
@@ -78,7 +81,7 @@ static DEFINE_SPINLOCK(target_list_lock);
/**
* struct netconsole_target - Represents a configured netconsole target.
* @list: Links this target into the target_list.
* @item: Links us into the configfs subsystem hierarchy.
* @np_state: Enabled / Disabled / SettingUp / Cleaning
* Visible from userspace (read-write) as "enabled".
@@ -175,10 +178,10 @@ static void deferred_netpoll_cleanup(struct work_struct *work)
nt = container_of(work, struct netconsole_target, cleanup_work);
netpoll_cleanup(&nt->np);
- spin_lock_irqsave(&target_list_lock, flags);
+ spin_lock_irqsave(&targets.lock, flags);
BUG_ON(nt->np_state != NETPOLL_CLEANING);
nt->np_state = NETPOLL_DISABLED;
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_unlock_irqrestore(&targets.lock, flags);
netconsole_target_put(nt);
}
@@ -365,7 +368,7 @@ static ssize_t store_enabled(struct netconsole_target *nt,
return enabled;
if (enabled) { /* 1 */
- spin_lock_irqsave(&target_list_lock, flags);
+ spin_lock_irqsave(&targets.lock, flags);
if (nt->np_state != NETPOLL_DISABLED)
goto busy;
else {
@@ -377,7 +380,7 @@ static ssize_t store_enabled(struct netconsole_target *nt,
* because there is a reference implicitly held by the
* caller of the store operation.
*/
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_unlock_irqrestore(&targets.lock, flags);
}
/*
@@ -387,34 +390,34 @@ static ssize_t store_enabled(struct netconsole_target *nt,
netpoll_print_options(&nt->np);
err = netpoll_setup(&nt->np);
- spin_lock_irqsave(&target_list_lock, flags);
+ spin_lock_irqsave(&targets.lock, flags);
if (err)
nt->np_state = NETPOLL_DISABLED;
else
nt->np_state = NETPOLL_ENABLED;
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_unlock_irqrestore(&targets.lock, flags);
if (err)
return err;
printk(KERN_INFO "netconsole: network logging started\n");
} else { /* 0 */
- spin_lock_irqsave(&target_list_lock, flags);
+ spin_lock_irqsave(&targets.lock, flags);
if (nt->np_state == NETPOLL_ENABLED)
nt->np_state = NETPOLL_CLEANING;
else if (nt->np_state != NETPOLL_DISABLED)
goto busy;
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_unlock_irqrestore(&targets.lock, flags);
netpoll_cleanup(&nt->np);
- spin_lock_irqsave(&target_list_lock, flags);
+ spin_lock_irqsave(&targets.lock, flags);
nt->np_state = NETPOLL_DISABLED;
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_unlock_irqrestore(&targets.lock, flags);
}
return strnlen(buf, count);
busy:
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_unlock_irqrestore(&targets.lock, flags);
return -EBUSY;
}
@@ -531,16 +534,16 @@ static ssize_t store_locked_##_name(struct netconsole_target *nt, \
{ \
unsigned long flags; \
ssize_t ret; \
- spin_lock_irqsave(&target_list_lock, flags); \
+ spin_lock_irqsave(&targets.lock, flags); \
if (nt->np_state != NETPOLL_DISABLED) { \
printk(KERN_ERR "netconsole: target (%s) is enabled, " \
"disable to update parameters\n", \
config_item_name(&nt->item)); \
- spin_unlock_irqrestore(&target_list_lock, flags); \
+ spin_unlock_irqrestore(&targets.lock, flags); \
return -EBUSY; \
} \
ret = store_##_name(nt, buf, count); \
- spin_unlock_irqrestore(&target_list_lock, flags); \
+ spin_unlock_irqrestore(&targets.lock, flags); \
return ret; \
}
@@ -549,9 +552,9 @@ static ssize_t show_locked_##_name(struct netconsole_target *nt, char *buf) \
{ \
unsigned long flags; \
ssize_t ret; \
- spin_lock_irqsave(&target_list_lock, flags); \
+ spin_lock_irqsave(&targets.lock, flags); \
ret = show_##_name(nt, buf); \
- spin_unlock_irqrestore(&target_list_lock, flags); \
+ spin_unlock_irqrestore(&targets.lock, flags); \
return ret; \
}
@@ -668,9 +671,9 @@ static struct config_item *make_netconsole_target(struct config_group *group,
config_item_init_type_name(&nt->item, name, &netconsole_target_type);
/* Adding, but it is disabled */
- spin_lock_irqsave(&target_list_lock, flags);
- list_add(&nt->list, &target_list);
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_lock_irqsave(&targets.lock, flags);
+ list_add(&nt->list, &targets.list);
+ spin_unlock_irqrestore(&targets.lock, flags);
return &nt->item;
}
@@ -681,9 +684,9 @@ static void drop_netconsole_target(struct config_group *group,
unsigned long flags;
struct netconsole_target *nt = to_target(item);
- spin_lock_irqsave(&target_list_lock, flags);
+ spin_lock_irqsave(&targets.lock, flags);
list_del(&nt->list);
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_unlock_irqrestore(&targets.lock, flags);
/*
* The target may have never been enabled, or was manually disabled
@@ -695,7 +698,7 @@ static void drop_netconsole_target(struct config_group *group,
if (nt->np_state == NETPOLL_ENABLED)
netpoll_cleanup(&nt->np);
- config_item_put(&nt->item);
+ netconsole_target_put(nt);
}
static struct configfs_group_operations netconsole_subsys_group_ops = {
@@ -722,7 +725,7 @@ static struct configfs_subsystem netconsole_subsys = {
/*
* Call netpoll_cleanup on this target asynchronously.
- * target_list_lock is required.
+ * targets.lock is required.
*/
static void defer_netpoll_cleanup(struct netconsole_target *nt)
{
@@ -746,8 +749,8 @@ static int netconsole_netdev_event(struct notifier_block *this,
event == NETDEV_BONDING_DESLAVE))
goto done;
- spin_lock_irqsave(&target_list_lock, flags);
- list_for_each_entry(nt, &target_list, list) {
+ spin_lock_irqsave(&targets.lock, flags);
+ list_for_each_entry(nt, &targets.list, list) {
if (nt->np_state == NETPOLL_ENABLED && nt->np.dev == dev) {
switch (event) {
case NETDEV_CHANGENAME:
@@ -763,7 +766,7 @@ static int netconsole_netdev_event(struct notifier_block *this,
}
}
}
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_unlock_irqrestore(&targets.lock, flags);
if (event == NETDEV_UNREGISTER || event == NETDEV_BONDING_DESLAVE)
printk(KERN_INFO "netconsole: network logging stopped, "
"interface %s %s\n", dev->name,
@@ -785,11 +788,11 @@ static void write_msg(struct console *con, const char *msg, unsigned int len)
const char *tmp;
/* Avoid taking lock and disabling interrupts unnecessarily */
- if (list_empty(&target_list))
+ if (list_empty(&targets.list))
return;
- spin_lock_irqsave(&target_list_lock, flags);
- list_for_each_entry(nt, &target_list, list) {
+ spin_lock_irqsave(&targets.lock, flags);
+ list_for_each_entry(nt, &targets.list, list) {
if (nt->np_state == NETPOLL_ENABLED
&& netif_running(nt->np.dev)) {
/*
@@ -807,7 +810,7 @@ static void write_msg(struct console *con, const char *msg, unsigned int len)
}
}
}
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_unlock_irqrestore(&targets.lock, flags);
}
static struct console netconsole = {
@@ -834,9 +837,9 @@ static int __init init_netconsole(void)
/* Dump existing printks when we register */
netconsole.flags |= CON_PRINTBUFFER;
- spin_lock_irqsave(&target_list_lock, flags);
- list_add(&nt->list, &target_list);
- spin_unlock_irqrestore(&target_list_lock, flags);
+ spin_lock_irqsave(&targets.lock, flags);
+ list_add(&nt->list, &targets.list);
+ spin_unlock_irqrestore(&targets.lock, flags);
}
}
@@ -864,7 +867,7 @@ fail:
* from the boot/module option exist here). Skipping the list
* lock is safe here, and netpoll_cleanup() will sleep.
*/
- list_for_each_entry_safe(nt, tmp, &target_list, list) {
+ list_for_each_entry_safe(nt, tmp, &targets.list, list) {
list_del(&nt->list);
free_param_target(nt);
}
@@ -888,7 +891,7 @@ static void __exit cleanup_netconsole(void)
* destroy them. Skipping the list lock is safe here, and
* netpoll_cleanup() will sleep.
*/
- list_for_each_entry_safe(nt, tmp, &target_list, list) {
+ list_for_each_entry_safe(nt, tmp, &targets.list, list) {
list_del(&nt->list);
free_param_target(nt);
}
next prev parent reply other threads:[~2010-11-08 20:32 UTC|newest]
Thread overview: 47+ messages / expand[flat|nested] mbox.gz Atom feed top
2010-11-08 20:31 [PATCH v2 00/23] netoops support Mike Waychison
2010-11-08 20:31 ` [PATCH v2 01/23] netconsole: Remove unneeded reference counting Mike Waychison
2010-11-08 20:31 ` [PATCH v2 02/23] netconsole: Introduce locking over the netpoll fields Mike Waychison
2010-11-08 20:31 ` [PATCH v2 03/23] netconsole: Introduce 'enabled' state-machine Mike Waychison
2010-11-08 20:32 ` [PATCH v2 04/23] netconsole: Call netpoll_cleanup() in process context Mike Waychison
2010-11-09 12:07 ` Neil Horman
2010-11-09 17:18 ` Mike Waychison
2010-11-09 19:33 ` Neil Horman
2010-11-08 20:32 ` Mike Waychison [this message]
2010-11-08 20:32 ` [PATCH v2 06/23] netconsole: Push configfs_subsystem into netpoll_targets Mike Waychison
2010-11-08 20:32 ` [PATCH v2 07/23] netconsole: Move netdev_notifier " Mike Waychison
2010-11-08 20:32 ` [PATCH v2 08/23] netconsole: Split out netpoll_targets init/exit Mike Waychison
2010-11-08 20:32 ` [PATCH v2 09/23] netconsole: Add pointer to netpoll_targets Mike Waychison
2010-11-08 20:32 ` [PATCH v2 10/23] netconsole: Rename netconsole_target -> netpoll_target Mike Waychison
2010-11-08 20:32 ` [PATCH v2 11/23] netconsole: Abstract away the subsystem name Mike Waychison
2010-11-08 20:32 ` [PATCH v2 12/23] netpoll: Introduce netpoll_target configs Mike Waychison
2010-11-09 3:30 ` Américo Wang
2010-11-09 4:27 ` Américo Wang
2010-11-09 8:34 ` Mike Waychison
2010-11-09 9:06 ` Américo Wang
2010-11-09 9:38 ` [RFC PATCH] configfs: make it not be a module any more Américo Wang
2010-11-09 14:20 ` [PATCH v2 12/23] netpoll: Introduce netpoll_target configs Greg KH
2010-11-09 17:24 ` Mike Waychison
2010-11-09 17:27 ` Greg KH
2010-11-08 20:32 ` [PATCH v2 13/23] netconsole: Move setting of default ports Mike Waychison
2010-11-08 20:32 ` [PATCH v2 14/23] netpoll: Move target code into netpoll_targets.c Mike Waychison
2010-11-08 20:33 ` [PATCH v2 15/23] Oops: Pass regs to oops_exit() Mike Waychison
2010-11-08 20:33 ` [PATCH v2 16/23] kmsg_dumper: Pass pt_regs along to dumpers Mike Waychison
2010-11-08 20:33 ` [PATCH v2 17/23] kmsg_dumper: Introduce a new 'SOFT' dump reason Mike Waychison
2010-11-09 5:49 ` KOSAKI Motohiro
2010-11-09 5:54 ` KOSAKI Motohiro
2010-11-08 20:33 ` [PATCH v2 18/23] sys-rq: Add option to soft dump Mike Waychison
2010-11-08 21:09 ` Randy Dunlap
2010-11-08 22:27 ` Mike Waychison
2010-11-08 22:31 ` Randy Dunlap
2010-11-08 20:33 ` [PATCH v2 19/23] netoops: add core functionality Mike Waychison
2010-11-08 20:33 ` [PATCH v2 20/23] netoops: Add x86 specific bits to packet headers Mike Waychison
2010-11-09 14:22 ` Neil Horman
2010-11-09 17:56 ` Mike Waychison
2010-11-08 20:33 ` [PATCH v2 21/23] netoops: Add user programmable fields to the netoops packet Mike Waychison
2010-11-08 20:33 ` [PATCH v2 22/23] netoops: Add one-shot mode Mike Waychison
2010-11-08 20:33 ` [PATCH v2 23/23] netoops: Add an interface to trigger various types of crashes Mike Waychison
2010-11-08 20:55 ` [PATCH v2 00/23] netoops support Matt Mackall
2010-11-08 21:20 ` David Miller
2010-11-08 21:43 ` Mike Waychison
2010-11-09 1:28 ` Andi Kleen
2010-11-09 4:25 ` Américo Wang
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20101108203205.22479.97761.stgit@crlf.mtv.corp.google.com \
--to=mikew@google.com \
--cc=adurbin@google.com \
--cc=akpm@linux-foundation.org \
--cc=chavey@google.com \
--cc=davem@davemloft.net \
--cc=greg@kroah.com \
--cc=linux-api@vger.kernel.org \
--cc=linux-kernel@vger.kernel.org \
--cc=mpm@selenic.com \
--cc=nhorman@tuxdriver.com \
--cc=simon.kagstrom@netinsight.net \
--cc=xiyou.wangcong@gmail.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -78,7 +81,7 @@ static DEFINE_SPINLOCK(target_list_lock);
/**
* struct netconsole_target - Represents a configured netconsole target.
- * @list: Links this target into the target_list.
+ * @list: Links this target into the netpoll_targets.list.
* @item: Links us into the configfs subsystem hierarchy.
* @np_state: Enabled / Disabled / SettingUp / Cleaning
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
Powered by JetHome